import { Constants } from "./Constants.js";
import { Game } from "./Game.js";
import { useState } from "./SuiWeb.js";

const saveGameKey = "connect-force-save";
const newGameClass = "new-game";
const saveGameClass = "save-game";
const loadGameClass = "load-game";
const undoClass = "undo-game";

/**
 * Gets a component which represents game.
 *
 * @returns {NodeDescriptor}
 * The rendered node.
 */
export function App()
{
    const [game, setGame] = /** @type {[Game, (updateFunction: (game: Game) => void) => void]} */ (useState("game", "game", new Game()));

    return [
        "div",
        {
            onclick: (event) =>
            {
                let target = /** @type {HTMLElement} */ (event.target);

                if (target.classList.contains(newGameClass))
                {
                    setGame(
                        (game) =>
                        {
                            game.reset();
                            return game;
                        });
                }
                else if (target.classList.contains(saveGameClass))
                {
                    localStorage.setItem(saveGameKey, JSON.stringify(game.dump()));
                }
                else if (target.classList.contains(loadGameClass))
                {
                    setGame(
                        (game) =>
                        {
                            game.load(JSON.parse(localStorage.getItem(saveGameKey)));
                            return game;
                        });
                }
                else if (target.classList.contains(undoClass))
                {
                    setGame(
                        (game) =>
                        {
                            game.undo();
                            return game;
                        });
                }
                else
                {
                    for (let y = 0; y < game.board.length; y++)
                    {
                        let rowLength = game.board[y].length;

                        for (let x = 0; x < rowLength; x++)
                        {
                            let node = document.querySelector(".board").children.item(y * rowLength + x);

                            if (node === target || node.contains(target))
                            {
                                setGame(
                                    (game) =>
                                    {
                                        game.addChip(x, y);
                                        return game;
                                    });
                            }
                        }
                    }
                }
            }
        },
        {
            className: `game ${game.winner ? `winner ${game.winner}` : ""}`
        },
        [
            Board,
            { board: game.board }
        ],
        [
            "div",
            { className: "log" },
            game.winner ?
                `Player ${Constants.PLAYER_NAMES[game.winner]} wins!` :
                `It's player "${Constants.PLAYER_NAMES[game.currentPlayer]}"s turn`
        ],
        [MenuBar]
    ];
}

/**
 * Renders an element which represents the specified {@link board `board`}.
 *
 * @param {{ board: Board }} board
 * The board represented in this element.
 *
 * @returns {NodeDescriptor}
 * The rendered element.
 */
export function Board({ board })
{
    console.log(board);
    let fields = board.flatMap((row) => row);

    return [
        "div",
        { className: "board" },
        ...fields.map(
            (field) =>
            {
                return /** @type {NodeDescriptor} */([Field, { field }]);
            }),
        ["div", { style: "clear: both;" }]
    ];
}

/**
 * Renders an element which represents the specified {@link field `field`}.
 *
 * @param {{ field: CellOwner }} field
 * The field to represent.
 *
 * @returns {NodeDescriptor}
 * The rendered element.
 */
export function Field({ field })
{
    return [
        "div",
        { className: "field" },
        ...(
            field !== "" ?
                [
                    /** @type {NodeDescriptor} */
                    ([
                        "div",
                        { className: `piece ${Constants.PLAYER_NAMES[field]}` }
                    ])
                ] :
                [])
    ];
}

/**
 * Renders a menu bar.
 *
 * @returns {NodeDescriptor}
 * The rendered element.
 */
export function MenuBar()
{
    return [
        "div",
        { className: "menu-bar" },
        [Button, { content: ["New Game", { className: newGameClass }] }],
        [Button, { content: ["Save Game", { className: saveGameClass }] }],
        [Button, { content: ["Load Game", { className: loadGameClass }] }],
        [Button, { content: ["Undo Last Move", { className: undoClass }] }]
    ];
}

/**
 * Renders a button.
 *
 * @param {{ content: ElementDescriptor[1][] }} content
 * The settings of the button.
 *
 * @returns {NodeDescriptor}
 * The rendered element.
 */
export function Button({ content })
{
    return [
        "button",
        ...content
    ];
}
